Package: survey
Title: Analysis of Complex Survey Samples
Description: Summary statistics, two-sample tests, rank tests, generalised linear models, cumulative link models, Cox models, loglinear models, and general maximum pseudolikelihood estimation for multistage stratified, cluster-sampled, unequally weighted survey samples. Variances by Taylor series linearisation or replicate weights. Post-stratification, calibration, and raking. Two-phase and multiphase subsampling designs. Graphics. PPS sampling without replacement. Small-area estimation. Dual-frame designs.
Version: 4.5-2
Authors@R: c(person(given = "\"Thomas", family = "Lumley\"", role = "cre", email = "t.lumley@auckland.ac.nz"),
		    person(given = "Peter", family = "Gao", role ="aut"),
		    person(given = "Ben", family = "Schneider", role= "aut"),
		    person(given = "Stas", family= "Kolenkikov", role="aut"))
Maintainer: "Thomas Lumley" <t.lumley@auckland.ac.nz>
License: GPL-2 | GPL-3
Depends: R (>= 4.1.0), grid, methods, Matrix, survival
Imports: stats, graphics, splines, lattice, minqa, numDeriv, mitools
        (>= 2.4), Rcpp (>= 0.12.8)
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: R.rsp, knitr
Suggests: foreign, MASS, KernSmooth, hexbin, RSQLite, quantreg,
        parallel, CompQuadForm, DBI, AER, SUMMER (>= 1.4.0), R.rsp,
        knitr, testthat (>= 3.0.0)
URL: http://r-survey.r-forge.r-project.org/survey/
Config/testthat/edition: 3
NeedsCompilation: yes
Author: "Thomas Lumley" [cre],
  Peter Gao [aut],
  Ben Schneider [aut],
  Stas Kolenkikov [aut]
Repository: R-Forge
Repository/R-Forge/Project: r-survey
Repository/R-Forge/Revision: 405
Repository/R-Forge/DateTimeStamp: 2026-07-02 23:52:25
Date/Publication: 2026-07-02 23:52:25
Packaged: 2026-07-03 00:05:08 UTC; rforge
