# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

arma_onestage <- function(Y, samp_unit_ids, strata_ids, strata_samp_sizes, strata_pop_sizes, singleton_method, use_singleton_method_for_domains, stage) {
    .Call('_survey_arma_onestage', PACKAGE = 'survey', Y, samp_unit_ids, strata_ids, strata_samp_sizes, strata_pop_sizes, singleton_method, use_singleton_method_for_domains, stage)
}

arma_multistage <- function(Y, samp_unit_ids, strata_ids, strata_samp_sizes, strata_pop_sizes, singleton_method, use_singleton_method_for_domains, use_only_first_stage, stage) {
    .Call('_survey_arma_multistage', PACKAGE = 'survey', Y, samp_unit_ids, strata_ids, strata_samp_sizes, strata_pop_sizes, singleton_method, use_singleton_method_for_domains, use_only_first_stage, stage)
}

